**==hrlev.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
      SUBROUTINE HRLEV(DELE,JMAX,B,V,n,IR,IMAX,zpe)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c DenSum: a code for calculating sums and densities of states.
c Copyright (C) 2003 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
C
C	CALCULATES ENERGY LEVELS FOR HINDERED ROTOR TO BE USED
C          WITH STERAB
C
c	DELE	= energy grain
c	JMAX	= index of Emax
c	B	= rotational constant (cm-1)
c	w	= vibrational frequency (cm-1)
c	V	= hindrance potential (cm-1)
c	n	= periodicity of hindrance potential
c	IR	= vector of energy level indices
c	IMAX	= ceiling of energy index array
c	ns	= state number (1,2,3,...)
c       zpe     = zero point energy of hindered rotor
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c    2/02 Modification: now, hindered rotors are counted as vibs in Whitten-Rabinovitch
c         parameter calculations. Zero point energy is calculated based on anharmonic
c         vibrations and from the hindered rotor subroutine.
c   Nov 01  Bug fix: previously, an extra state was included as array 
c           element IR(0), exceeding the array bounds. This had no
c           effect on most platforms, but crashed on others.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      EXTERNAL STERAB , GAM , WRAB , WRDEN , ALNGAM
      DIMENSION IR(20001)
      SAVE 
 
      EEmax = (JMAX-1)*DELE

      IF (V .GT. 1.0d-005) THEN                ! Hindered rotor
         w = n*SQRT(B*V)      ! vibrational frequency
         r = V/w
         Enet = 0.0
         ns = 1
         DO WHILE ( Enet .LE. EEmax )

           nv = INT((ns-1)/n)
           Evib = w*(nv+0.5) - w*(1+2.0*nv+2.0*nv*nv)/(16.0*r)
 
           j = ns/2 
           IF ( j.GT.r*n/2 .AND. r.GT.0.0 ) THEN
               Term = (r**4)*(n*n)*B/(8.0*((2.0*j/n)**2-1.0))
           IF((2.0*j/n).eq.1)    Term=-0.1100                  ! =1-r^2-r^4/8+r^6/64-r^8/1536   when (2.0*j/n)=1
           IF((2.0*j/n)**2.eq.2) Term=4.3615                   ! =4+5/12r^4-763/13824r^8        when (2.0*j/n)=2
             ELSE
              Term = 0.0
           ENDIF

           Erot = B*j*j + V/2.0 + Term
 
            IF ( Erot.GT.1.5*V ) THEN
              S = 1.0
             ELSE
              S = 0.5*(1.0+TANH((Evib-V)/(0.2*V)))
            ENDIF
 
            Enet = (1.0-S)*Evib + S*Erot
 
            IF ( ns.EQ.1 ) zpe = Enet
            RR = (Enet-zpe)/DELE   ! relative to zpe
            IRR = IDNINT(RR)         ! Nearest integer: number of grains
c            IRR = INT(RR) + 1        ! Truncate: number of grains

            IF ( ns .GT. 1 ) THEN
               IR(ns-1) = IRR          ! number of grains
            ENDIF
              
            IMAX = ns - 1
            ns = ns + 1
         ENDDO
       ELSE                        ! Free Rotor
        zpe = 0.0
        j = 0
        DO WHILE ( Enet .LE. EEmax ) 
           j = j + 1
           Enet = B*j*j
           RR = Enet/DELE   ! relative to zpe 
           IRR = IDNINT(RR)         ! Nearest integer: number of grains
c           IRR = INT(RR) + 1        ! Truncate: number of grains
           IR(j) = IRR              ! number of grains 
         ENDDO
       ENDIF

      RETURN
      END
