**==rs.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
c
c     EISPACK SUBROUTINES
c
c     http://www.netlib.org/eispack/
c
c     B.T. Smith, J.M. Boyle, J.J. Dongarra, B.S. Garbow, Y. Ikebe, 
c     V.C. Klema, and C.B. Moler. Matrix Eigensystem Routines - 
c     EISPACK Guide, volume 6 of Lecture Notes in Computer Science, 
c     Springer-Verlag, Berlin, 1976.
c
c     B.S. Garbow, J.M. Boyle, J.J. Dongarra, and C.B. Moler. Matrix 
c     Eigensystem Routines - EISPACK Guide Extension, volume 51 of
c     Lecture Notes in Computer Science, Springer-Verlag, Berlin, 
c     1977.
c    
c
c
      SUBROUTINE rs(nm,n,a,w,matz,z,fv1,fv2,ierr)
c
      INTEGER n , nm , ierr , matz
      DOUBLE PRECISION a(nm,n) , w(n) , z(nm,n) , fv1(n) , fv2(n)
c
c     this subroutine calls the recommended sequence of
c     subroutines from the eigensystem subroutine package (eispack)
c     to find the eigenvalues and eigenvectors (if desired)
c     of a real symmetric matrix.
c
c     on input
c
c        nm  must be set to the row dimension of the two-dimensional
c        array parameters as declared in the calling program
c        dimension statement.
c
c        n  is the order of the matrix  a.
c
c        a  contains the real symmetric matrix.
c
c        matz  is an integer variable set equal to zero if
c        only eigenvalues are desired.  otherwise it is set to
c        any non-zero integer for both eigenvalues and eigenvectors.
c
c     on output
c
c        w  contains the eigenvalues in ascending order.
c
c        z  contains the eigenvectors if matz is not zero.
c
c        ierr  is an integer output variable set equal to an error
c           completion code described in the documentation for tqlrat
c           and tql2.  the normal completion code is zero.
c
c        fv1  and  fv2  are temporary storage arrays.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      IF ( n.GT.nm ) THEN
         ierr = 10*n
c
      ELSEIF ( matz.NE.0 ) THEN
c     .......... find both eigenvalues and eigenvectors ..........
         CALL tred2(nm,n,a,w,fv1,z)
         CALL tql2(nm,n,w,fv1,z,ierr)
      ELSE
c     .......... find eigenvalues only ..........
         CALL tred1(nm,n,a,w,fv1,fv2)
*  tqlrat encounters catastrophic underflow on the Vax
*     call  tqlrat(n,w,fv2,ierr)
         CALL tql1(n,w,fv1,ierr)
      ENDIF
      RETURN
      END
**==pythag.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
 
      DOUBLE PRECISION FUNCTION pythag(a,b)
      DOUBLE PRECISION a , b
c
c     finds dsqrt(a**2+b**2) without overflow or destructive underflow
c
      DOUBLE PRECISION p , r , s , t , u
      p = dmax1(dabs(a),dabs(b))
      IF ( abs(p).GT.0.0D-06 ) THEN
         r = (dmin1(dabs(a),dabs(b))/p)**2
 50      t = 4.0D0 + r
         IF ( abs(t-4.0D+00).GT.1.D-06 ) THEN
            s = r/t
            u = 1.0D0 + 2.0D0*s
            p = u*p
            r = (s/u)**2*r
            GOTO 50
         ENDIF
      ENDIF
      pythag = p
      RETURN
      END
**==tql1.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
 
      SUBROUTINE tql1(n,d,e,ierr)
c
      INTEGER i , j , l , m , n , ii , l1 , l2 , mml , ierr
      DOUBLE PRECISION d(n) , e(n)
      DOUBLE PRECISION c , c2 , c3 , dl1 , el1 , f , g , h , p , r , s , 
     &                 s2 , tst1 , tst2 , pythag
c
c     this subroutine is a translation of the algol procedure tql1,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues of a symmetric
c     tridiagonal matrix by the ql method.
c
c     on input
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct and
c          ordered for indices 1,2,...ierr-1, but may not be
c          the smallest eigenvalues.
c
c        e has been destroyed.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      IF ( n.NE.1 ) THEN
c
         DO 50 i = 2 , n
            e(i-1) = e(i)
 50      CONTINUE
c
         f = 0.0D0
         tst1 = 0.0D0
         e(n) = 0.0D0
c
         DO 150 l = 1 , n
            j = 0
            h = dabs(d(l)) + dabs(e(l))
            IF ( tst1.LT.h ) tst1 = h
c     .......... look for small sub-diagonal element ..........
            DO 60 m = l , n
               tst2 = tst1 + dabs(e(m))
               IF ( abs(tst2-tst1).LE.1.0D-06 ) GOTO 80
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
 60         CONTINUE
c
 80         IF ( m.NE.l ) THEN
 90            IF ( j.EQ.30 ) GOTO 200
               j = j + 1
c     .......... form shift ..........
               l1 = l + 1
               l2 = l1 + 1
               g = d(l)
               p = (d(l1)-g)/(2.0D0*e(l))
               r = pythag(p,1.0D0)
               d(l) = e(l)/(p+dsign(r,p))
               d(l1) = e(l)*(p+dsign(r,p))
               dl1 = d(l1)
               h = g - d(l)
               IF ( l2.LE.n ) THEN
c
                  DO 95 i = l2 , n
                     d(i) = d(i) - h
 95               CONTINUE
               ENDIF
c
               f = f + h
c     .......... ql transformation ..........
               p = d(m)
               c = 1.0D0
               c2 = c
               el1 = e(l1)
               s = 0.0D0
               mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
               DO 100 ii = 1 , mml
                  c3 = c2
                  c2 = c
                  s2 = s
                  i = m - ii
                  g = c*e(i)
                  h = c*p
                  r = pythag(p,e(i))
                  e(i+1) = s*r
                  s = e(i)/r
                  c = p/r
                  p = c*d(i) - s*g
                  d(i+1) = h + s*(c*g+s*d(i))
 100           CONTINUE
c
               p = -s*s2*c3*el1*e(l)/dl1
               e(l) = s*p
               d(l) = c*p
               tst2 = tst1 + dabs(e(l))
               IF ( tst2.GT.tst1 ) GOTO 90
            ENDIF
            p = d(l) + f
c     .......... order eigenvalues ..........
            IF ( l.NE.1 ) THEN
c     .......... for i=l step -1 until 2 do -- ..........
               DO 110 ii = 2 , l
                  i = l + 2 - ii
                  IF ( p.GE.d(i-1) ) GOTO 120
                  d(i) = d(i-1)
 110           CONTINUE
            ENDIF
c
            i = 1
 120        d(i) = p
c
 150     CONTINUE
      ENDIF
      GOTO 300
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 200  ierr = l
 300  RETURN
      END
**==tql2.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
 
 
      SUBROUTINE tql2(nm,n,d,e,z,ierr)
c
      INTEGER i , j , k , l , m , n , ii , l1 , l2 , nm , mml , ierr
      DOUBLE PRECISION d(n) , e(n) , z(nm,n)
      DOUBLE PRECISION c , c2 , c3 , dl1 , el1 , f , g , h , p , r , s , 
     &                 s2 , tst1 , tst2 , pythag
c
c     this subroutine is a translation of the algol procedure tql2,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues and eigenvectors
c     of a symmetric tridiagonal matrix by the ql method.
c     the eigenvectors of a full symmetric matrix can also
c     be found if  tred2  has been used to reduce this
c     full matrix to tridiagonal form.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c        z contains the transformation matrix produced in the
c          reduction by  tred2, if performed.  if the eigenvectors
c          of the tridiagonal matrix are desired, z must contain
c          the identity matrix.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct but
c          unordered for indices 1,2,...,ierr-1.
c
c        e has been destroyed.
c
c        z contains orthonormal eigenvectors of the symmetric
c          tridiagonal (or full) matrix.  if an error exit is made,
c          z contains the eigenvectors associated with the stored
c          eigenvalues.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      IF ( n.NE.1 ) THEN
c
         DO 50 i = 2 , n
            e(i-1) = e(i)
 50      CONTINUE
c
         f = 0.0D0
         tst1 = 0.0D0
         e(n) = 0.0D0
c
         DO 150 l = 1 , n
            j = 0
            h = dabs(d(l)) + dabs(e(l))
            IF ( tst1.LT.h ) tst1 = h
c     .......... look for small sub-diagonal element ..........
            DO 60 m = l , n
               tst2 = tst1 + dabs(e(m))
               IF ( abs(tst2-tst1).LE.1.0D-06 ) GOTO 80
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
 60         CONTINUE
c
 80         IF ( m.NE.l ) THEN
 90            IF ( j.EQ.30 ) GOTO 300
               j = j + 1
c     .......... form shift ..........
               l1 = l + 1
               l2 = l1 + 1
               g = d(l)
               p = (d(l1)-g)/(2.0D0*e(l))
               r = pythag(p,1.0D0)
               d(l) = e(l)/(p+dsign(r,p))
               d(l1) = e(l)*(p+dsign(r,p))
               dl1 = d(l1)
               h = g - d(l)
               IF ( l2.LE.n ) THEN
c
                  DO 95 i = l2 , n
                     d(i) = d(i) - h
 95               CONTINUE
               ENDIF
c
               f = f + h
c     .......... ql transformation ..........
               p = d(m)
               c = 1.0D0
               c2 = c
               el1 = e(l1)
               s = 0.0D0
               mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
               DO 110 ii = 1 , mml
                  c3 = c2
                  c2 = c
                  s2 = s
                  i = m - ii
                  g = c*e(i)
                  h = c*p
                  r = pythag(p,e(i))
                  e(i+1) = s*r
                  s = e(i)/r
                  c = p/r
                  p = c*d(i) - s*g
                  d(i+1) = h + s*(c*g+s*d(i))
c     .......... form vector ..........
                  DO 100 k = 1 , n
                     h = z(k,i+1)
                     z(k,i+1) = s*z(k,i) + c*h
                     z(k,i) = c*z(k,i) - s*h
 100              CONTINUE
c
 110           CONTINUE
c
               p = -s*s2*c3*el1*e(l)/dl1
               e(l) = s*p
               d(l) = c*p
               tst2 = tst1 + dabs(e(l))
               IF ( tst2.GT.tst1 ) GOTO 90
            ENDIF
            d(l) = d(l) + f
 150     CONTINUE
c     .......... order eigenvalues and eigenvectors ..........
         DO 200 ii = 2 , n
            i = ii - 1
            k = i
            p = d(i)
c
            DO 160 j = ii , n
               IF ( d(j).LT.p ) THEN
                  k = j
                  p = d(j)
               ENDIF
 160        CONTINUE
c
            IF ( k.NE.i ) THEN
               d(k) = d(i)
               d(i) = p
c
               DO 170 j = 1 , n
                  p = z(j,i)
                  z(j,i) = z(j,k)
                  z(j,k) = p
 170           CONTINUE
            ENDIF
c
c
 200     CONTINUE
      ENDIF
      GOTO 400
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 300  ierr = l
 400  RETURN
      END
**==tred1.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
 
 
      SUBROUTINE tred1(nm,n,a,d,e,e2)
c
      INTEGER i , j , k , l , n , ii , nm , jp1
      DOUBLE PRECISION a(nm,n) , d(n) , e(n) , e2(n)
      DOUBLE PRECISION f , g , h , scale
c
c     this subroutine is a translation of the algol procedure tred1,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix
c     to a symmetric tridiagonal matrix using
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        a contains information about the orthogonal trans-
c          formations used in the reduction in its strict lower
c          triangle.  the full upper triangle of a is unaltered.
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        e2 contains the squares of the corresponding elements of e.
c          e2 may coincide with e if the squares are not needed.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      DO 100 i = 1 , n
         d(i) = a(n,i)
         a(n,i) = a(i,i)
 100  CONTINUE
c     .......... for i=n step -1 until 1 do -- ..........
      DO 200 ii = 1 , n
         i = n + 1 - ii
         l = i - 1
         h = 0.0D0
         scale = 0.0D0
         IF ( l.GE.1 ) THEN
c     .......... scale row (algol tol then not needed) ..........
            DO 120 k = 1 , l
               scale = scale + dabs(d(k))
 120        CONTINUE
c
            IF ( abs(scale-0.0D0).GT.1.D-06 ) THEN
c
               DO 130 k = 1 , l
                  d(k) = d(k)/scale
                  h = h + d(k)*d(k)
 130           CONTINUE
c
               e2(i) = scale*scale*h
               f = d(l)
               g = -dsign(dsqrt(h),f)
               e(i) = scale*g
               h = h - f*g
               d(l) = f - g
               IF ( l.NE.1 ) THEN
c     .......... form a*u ..........
                  DO 135 j = 1 , l
                     e(j) = 0.0D0
 135              CONTINUE
c
                  DO 140 j = 1 , l
                     f = d(j)
                     g = e(j) + a(j,j)*f
                     jp1 = j + 1
                     IF ( l.GE.jp1 ) THEN
c
                        DO 136 k = jp1 , l
                           g = g + a(k,j)*d(k)
                           e(k) = e(k) + a(k,j)*f
 136                    CONTINUE
                     ENDIF
c
                     e(j) = g
 140              CONTINUE
c     .......... form p ..........
                  f = 0.0D0
c
                  DO 145 j = 1 , l
                     e(j) = e(j)/h
                     f = f + e(j)*d(j)
 145              CONTINUE
c
                  h = f/(h+h)
c     .......... form q ..........
                  DO 150 j = 1 , l
                     e(j) = e(j) - h*d(j)
 150              CONTINUE
c     .......... form reduced a ..........
                  DO 155 j = 1 , l
                     f = d(j)
                     g = e(j)
c
                     DO 152 k = j , l
                        a(k,j) = a(k,j) - f*e(k) - g*d(k)
 152                 CONTINUE
c
 155              CONTINUE
               ENDIF
c
               DO 160 j = 1 , l
                  f = d(j)
                  d(j) = a(l,j)
                  a(l,j) = a(i,j)
                  a(i,j) = f*scale
 160           CONTINUE
               GOTO 200
            ELSE
c
               DO 170 j = 1 , l
                  d(j) = a(l,j)
                  a(l,j) = a(i,j)
                  a(i,j) = 0.0D0
 170           CONTINUE
            ENDIF
         ENDIF
c
         e(i) = 0.0D0
         e2(i) = 0.0D0
c
 200  CONTINUE
c
      RETURN
      END
**==tred2.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
 
      SUBROUTINE tred2(nm,n,a,d,e,z)
c
      INTEGER i , j , k , l , n , ii , nm , jp1
      DOUBLE PRECISION a(nm,n) , d(n) , e(n) , z(nm,n)
      DOUBLE PRECISION f , g , h , hh , scale
c
c     this subroutine is a translation of the algol procedure tred2,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix to a
c     symmetric tridiagonal matrix using and accumulating
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        z contains the orthogonal transformation matrix
c          produced in the reduction.
c
c        a and z may coincide.  if distinct, a is unaltered.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      DO 100 i = 1 , n
c
         DO 50 j = i , n
            z(j,i) = a(j,i)
 50      CONTINUE
c
         d(i) = a(n,i)
 100  CONTINUE
c
      IF ( n.NE.1 ) THEN
c     .......... for i=n step -1 until 2 do -- ..........
         DO 200 ii = 2 , n
            i = n + 2 - ii
            l = i - 1
            h = 0.0D0
            scale = 0.0D0
            IF ( l.GE.2 ) THEN
c     .......... scale row (algol tol then not needed) ..........
               DO 110 k = 1 , l
                  scale = scale + dabs(d(k))
 110           CONTINUE
c
               IF ( abs(scale-0.0D0).GT.1.D-06 ) THEN
c
                  DO 115 k = 1 , l
                     d(k) = d(k)/scale
                     h = h + d(k)*d(k)
 115              CONTINUE
c
                  f = d(l)
                  g = -dsign(dsqrt(h),f)
                  e(i) = scale*g
                  h = h - f*g
                  d(l) = f - g
c     .......... form a*u ..........
                  DO 120 j = 1 , l
                     e(j) = 0.0D0
 120              CONTINUE
c
                  DO 125 j = 1 , l
                     f = d(j)
                     z(j,i) = f
                     g = e(j) + z(j,j)*f
                     jp1 = j + 1
                     IF ( l.GE.jp1 ) THEN
c
                        DO 122 k = jp1 , l
                           g = g + z(k,j)*d(k)
                           e(k) = e(k) + z(k,j)*f
 122                    CONTINUE
                     ENDIF
c
                     e(j) = g
 125              CONTINUE
c     .......... form p ..........
                  f = 0.0D0
c
                  DO 130 j = 1 , l
                     e(j) = e(j)/h
                     f = f + e(j)*d(j)
 130              CONTINUE
c
                  hh = f/(h+h)
c     .......... form q ..........
                  DO 135 j = 1 , l
                     e(j) = e(j) - hh*d(j)
 135              CONTINUE
c     .......... form reduced a ..........
                  DO 140 j = 1 , l
                     f = d(j)
                     g = e(j)
c
                     DO 136 k = j , l
                        z(k,j) = z(k,j) - f*e(k) - g*d(k)
 136                 CONTINUE
c
                     d(j) = z(l,j)
                     z(i,j) = 0.0D0
 140              CONTINUE
                  GOTO 180
               ENDIF
            ENDIF
            e(i) = d(l)
c
            DO 160 j = 1 , l
               d(j) = z(l,j)
               z(i,j) = 0.0D0
               z(j,i) = 0.0D0
c
 160        CONTINUE
c
 180        d(i) = h
 200     CONTINUE
c     .......... accumulation of transformation matrices ..........
         DO 250 i = 2 , n
            l = i - 1
            z(n,l) = z(l,l)
            z(l,l) = 1.0D0
            h = d(i)
            IF ( abs(h).LE.1.0D-06 ) THEN
c
               DO 210 k = 1 , l
                  d(k) = z(k,i)/h
 210           CONTINUE
c
               DO 230 j = 1 , l
                  g = 0.0D0
c
                  DO 215 k = 1 , l
                     g = g + z(k,i)*z(k,j)
 215              CONTINUE
c
                  DO 220 k = 1 , l
                     z(k,j) = z(k,j) - g*d(k)
 220              CONTINUE
 230           CONTINUE
            ENDIF
c
            DO 240 k = 1 , l
               z(k,i) = 0.0D0
 240        CONTINUE
c
 250     CONTINUE
      ENDIF
c
      DO 300 i = 1 , n
         d(i) = z(n,i)
         z(n,i) = 0.0D0
 300  CONTINUE
c
      z(n,n) = 1.0D0
      e(1) = 0.0D0
      RETURN
      END
