**==colstep.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      DOUBLE PRECISION FUNCTION COLSTEP(Mol,E,Temp)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2005 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c      REVISIONS
c      12/2004  General overhaul to improve speed and accuracy, especially at 
c               low energies.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
C
C     FROM RANDOM NUMBER RON AND ENERGY E COMPUTE A COLLISIONAL
C     STEP SIZE FROM THE NORMALIZED CUMULATIVE DISTRIBUTION 
C     FUNCTION CORRESPONDING TO THE COLLISIONAL DOWN-STEP MODEL:
C
C	IFLAG2 = -1 FOR DOWN-STEPS
C	IFLAG2 = +1 FOR UP-STEPS
C
c	Step-size distribution given in FUNCTION PDOWN(E,EE,Mol,Temp,Hs)
c
c		Mol	= index of molecule
c		Temp	= translational temperature
c	 	E	= initial energy
c		EE	= final energy
c		DC	= coefficients for stepsize distribution
c		Hs	= characteristic stepsize at energy E (calculated by PDOWN
c
c	Note that COLLUP and CNORM are 'double arrays' with the first
c	imax1+1 elements starting at E = 0 with Egrain1 spacing; the remaining
c	elements are at Egrain2 spacing, starting at E = 0.
c
c	Coefficients stored in array DC
c	Probability of up-steps stored in COLLUP(MaxWell,Isize)
c	Log of Normalization factor (up + down) stored in CNORM(MaxWell,Isize)
c
c	ERROR	= Convergence criterion
c	EDIFF 	= Minimum integration interval (cm-1)
c	FRACT  	= Step-size relative to 'Hs'
c	ITERS  	= Iterations on up-steps
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare3.inc'
 
      DOUBLE PRECISION PDOWN , DENVR , Temp , CNORMI , CNORMJ , 
     &                 DOUBNTERP , RATIO , CD , X
 
      DOUBLE PRECISION UP , RON , TOT , ADOWN , AUP , B , RAN1 , a , 
     &                 AA , Ht , arg , DELMIN , ERROR , TEST , 
     &                 TESTH , CC
 
      INTEGER          Mol , IFLAG2 , IFLAG3 , Igg , Ntrap , JK , II
 
      DOUBLE PRECISION DENI , E , EE , Hs , PROB , H , DENJ , TERM , 
     &                 SPAN , TLAST , Tiggp , Tiggm
 
      EXTERNAL         DENVR , PDOWN , RAN1
 
      SAVE 

      IF ( E .GT. Emax2 ) THEN
        WRITE(*,*) '**********************************************'
        WRITE(*,*) ' FATAL ERROR at point #1 in colstep.f!'
        WRITE(*,*) ' E is greater than Emax2'
        WRITE(*,*) ' Suggest you increase Emax2'
        WRITE(*,*) '**********************************************'
        STOP 
      ENDIF

      Statest = 0.5d+00/Egrain1

      DENI = DENVR(Mol,E)                               ! DENI = Density at E
      CC = DOUBNTERP(E,Mol,CNORM)
      CNORMI = EXP( CC )                                ! CNORMI = Normalization at E
 
      PROB = PDOWN(E,E,Mol,Temp,Hs)     ! Evaluate to get initial Hs

      UP = DOUBNTERP(E,Mol,COLLUP)      ! UP = COLLUP at E = probability of an up-step
      IF ( UP .GT. 1.0d+00 ) UP = 1.0d+00

      RON = RAN1(IDUM)
      IF ( RON.LE.UP ) THEN             ! Select up-steps, or down-steps
         IFLAG2 = 1                     ! Up-step
        ELSE
         IFLAG2 = -1                    ! Down-step
      ENDIF

      IF ( E .LE. 0.0 ) IFLAG2 = 1
 
         ERROR = ERRo

c ******************** Down steps ***************
      IF ( IFLAG2.EQ.-1 ) THEN                  !  Integrate over down steps
 
         DELMIN = 5.*Hs                         !  5*<dE>

         CD = (1.d+00-ERROR)*CNORMI*(1.d+00-UP)
 50      RON = RAN1(IDUM)
         ADOWN = RON*CD !  Random-number-selected integral of down-steps

         TLAST = 1.0d+00                        ! 'old' TERM
         TOT = 0.0
         EE = E
         TEST = 1.0 
 
         DO WHILE (TOT.LT.ADOWN .AND. EE.GE.Egrain1 .AND. TEST.GT.ERROR)
            IF ( EE.GT.Emax1 ) THEN
               H = FRACT*Hs*(Temp/hor)/(Hs + (Temp/hor))    ! to ensure same step size for both up and down steps
               H = MIN( H , EE-Emax1 )
             ELSE
               H = Egrain1
            ENDIF

            IF ( H.GT.EE ) H = EE
            EE = EE - H
            DENJ = DENVR(Mol,EE)
            IF ( DENJ.GE.Statest ) THEN
               PROB = PDOWN(E,EE,Mol,Temp,Hs)   ! E > EE
               TERM = PROB
              ELSE
               TERM = 0.0D+00
            ENDIF
 
            IF ( EE .GT. Emax1 ) THEN
               Ntrap = 0
               a = -LOG(TERM/TLAST)/H                           ! Exponential form
               AA = TLAST                                       ! Exponential form
               SPAN = (AA/a)*(1.0D+00-EXP(-a*H))                ! Exponential form
              ELSE
               Ntrap = 1
               SPAN = 0.5D+00*H*( TERM + TLAST )                ! Trapezoidal Rule
            ENDIF
 
            TOT = TOT + SPAN                    ! Down-step normalization integral
 
            TLAST = TERM                        ! Old value for TERM
 
            IF ( (E-EE).GT.DELMIN .AND. SPAN.GT.0.0 ) TEST = SPAN/TOT
 
         ENDDO
 
         IF ( TOT.LT.ADOWN ) THEN       ! Invoked when estimated CNORMI is too large
            CD = TOT*(1.0d+00-ERROR)    ! Use new cumulative sum
            GOTO 50                     ! Repeat search with a fresh random number
         ENDIF
c                                       Begin correction for overshoot ( TOT > ADOWN )
         IF ( Ntrap .EQ. 1) THEN               ! Trapezpidal rule
             EE = EE + (TOT - ADOWN)*H/SPAN
           ELSE                                ! Exponential form
             arg = 1.0D+00 - a*( ADOWN - TOT + SPAN )/AA
             Ht = -LOG( arg )/a
             EE = EE + H - Ht           ! Correction for overshoot
          ENDIF

c ******************** Up steps ***************
      ELSE                                              ! Up-collisions
 
         CD = (1.d+00-ERROR)*CNORMI*UP
100      RON = RAN1(IDUM)
         AUP = RON*CD                   !  Random number-selected
         PROB = PDOWN(E,E,Mol,Temp,Hs)  ! Evaluate to get initial Hs
         DELMIN = Viblo(Mol) + 5.0*MIN(Hs, Temp/hor)
         TLAST = 1.0d+00                                ! 'old' TERM
         TOT = 0.0d+00
         EE = E
         TEST = 1.0
 
         DO WHILE ( TEST.GT.ERROR )
            IF ( EE .GE. Emax1 ) THEN
               H = FRACT*Hs*(Temp/hor)/(Hs + (Temp/hor))
              ELSE
               H = Egrain1
            ENDIF
            EE = EE + H
            DENJ = DENVR(Mol,EE)
            CC = DOUBNTERP(EE,Mol,CNORM)
            CNORMJ = EXP( CC )                          ! Normalization stored as log
            IF ( DENJ.GE.Statest .AND. CNORMJ.GT.0.0 )
     &           THEN
               B = EXP(-(EE-E)*hor/Temp)                ! EE > E
               PROB = PDOWN(EE,E,Mol,Temp,Hs)           ! EE > E
               RATIO = (CNORMI/CNORMJ)*(DENJ/DENI)
               TERM = B*PROB*RATIO
              ELSE
               TERM = 0.0D+00
            ENDIF
 
            IF ( E-H .GE. Emax1 ) THEN
               Ntrap = 0
               a = -LOG(TERM/TLAST)/H                           ! Exponential form
               AA = TLAST                                       ! Exponential form
               SPAN = (AA/a)*(1.0D+00-EXP(-a*H))                ! Exponential form
              ELSE
               Ntrap = 1
               SPAN = 0.5D+00*H*( TERM + TLAST )                ! Trapezoidal Rule
            ENDIF
 
            TLAST = TERM                        ! Old value for TERM
            TOT = TOT + SPAN                    ! Up-step normalization integral
 
            IF ( (EE-E).GT.DELMIN .AND. SPAN.GT.0.0 ) TEST = SPAN/TOT
            IF ( TOT .GE. AUP ) TEST = 0.0

      IF ( EE .GT. Emax2 ) THEN
        WRITE(*,*) '**********************************************'
        WRITE(*,*) ' FATAL ERROR at point #2 in colstep.f!'
        WRITE(*,*) ' EE is greater than Emax2'
        WRITE(*,*) ' Suggest you increase Emax2'
        WRITE(*,*) '**********************************************'
        STOP 
      ENDIF

         ENDDO
 
         IF ( TOT.LT.AUP ) THEN                 ! Invoked when estimated CNORMI was too large
            CD = TOT*(1.0-ERROR)                ! Use new cumulative sum
            GOTO 100                            ! Repeat search
         ENDIF

c                                       Begin correction for overshoot ( TOT > AUP )
         IF ( Ntrap .EQ. 1) THEN               ! Trapezpidal rule
             EE = EE - (TOT - AUP)*H/SPAN
           ELSE                                ! Exponential form
             arg = 1.0D+00 - a*( AUP - TOT + SPAN )/AA
             Ht = -LOG( arg )/a
             EE = EE - H +  Ht           ! Correction for overshoot
          ENDIF

      ENDIF
 
      IF ( EE.LT.0.0 ) EE = 0.0d+00
      IF ( EE.GT.Emax2 ) EE = Emax2

      IF ( EE .LE. Emax1) THEN               ! align with energy grains
        Igg = NINT( EE/Egrain1 )
        EE = Egrain1*Igg
        IF ( DENVR(Mol,EE) .LT. Statest ) THEN  ! if a state is not present, move up or down one grain
           Tiggp = DENVR(Mol,EE+Egrain1)
           Tiggm = DENVR(Mol,EE-Egrain1)
           IF ( Tiggp .GT. Tiggm ) THEN
               EE = EE + Egrain1
             ELSE
               EE = EE - Egrain1
           ENDIF
        ENDIF
        IF ( EE.LT.0.0 ) EE = 0.0d+00
      ENDIF
 
      COLSTEP = EE - E


      RETURN
      END
