c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2007 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

      SUBROUTINE QKinf(Mol,nchann,tx,Kinf,Ainf,Einf,klj,kosc)

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , i , IE , Itop , mm, i2
      DOUBLE PRECISION Q , RK , TT , E , D , DENVR , Kinf , Ainf , 
     &                 Einf , K , tx , Et , arg , Y1 , Y2 , Y3 , Y , 
     &                 X , Estep, RR, QQ , Qlast , Rlast , TTSTEP , 
     &                 TS , EC
      DOUBLE PRECISION klj, Qts, QQts, Qtslast, DENTS, E2, Qcheck
      DOUBLE PRECISION kosc(MaxWell,MaxChan)
      DIMENSION K(3), TS(3)
      SAVE 
c
c	Calculates high pressure limiting unimolecular rate constant according to
c         Equation (4.30) of Robinson & Holbrook, "Unimolecular Reactions"
c         (Wiley-Interscience, London, 1972), p. 90.
c        This is the same as equation 3.30 in Holbrook, Pilling, and Robertson,
c         "Unimolecular Reactions. Second Edition" (John Wiley & Sons, Chichester, 
c         1996), p. 64.
c
c	Mol	= index of molecule
c	nchann	= index of rxn channel
c	tx	= translational temperature
c	Kinf	= infinite pressure rate constant [Robinson & Holbrook Eq. 4.30]
c	Ainf	= A-factor for kinf
c	Einf	= Activation energy for kinf
c
c
c     Centrifugal correction at temperature = tx
c

      EC =  Eor(Mol,nchann)                       ! Includes centrifugal correction

      TTSTEP = 0.001d+00/tx                       ! Step size in 1/Temp
      TT = 1.0d+00/tx-2.d+00*TTSTEP               ! 1/Temp

      DO 1000 mm = 1,3
        K(mm) = 0.0d+00
        RK = 0.0d+00
        Q  = 0.0d+00
        Qts = 0.0d+00
        Qlast = exp(Dens(Mol,1))
        Qtslast = DENVR(Mol,EC)
        Rlast = 0.0d+00

        TT = TT + TTSTEP                          ! 1/T
        TS(mm) = TT                               ! 1/T

        Estep = Egrain1
        E = 0.0d+00
      DO 100 i = 2 , imax1                        
         E = E + Estep                            ! summation continues till end of first part of double array
         QQ = exp(Dens(Mol,i)-E*hor*TT)           ! molecule partition fxn;  density stored as log
         arg = E + EC
         D = DENVR(Mol,arg)
         RR = D*exp(Rate(Mol,nchann,i)-E*hor*TT)  ! thermal rate constant; k(E) stored as log
         QQts=D*exp(-E*hor*TT)                    ! partition function for active energy;
         Q  = Q  + 0.5*Estep*(QQ + Qlast)         ! Trapezoidal Rule
         RK = RK + 0.5*Estep*(RR + Rlast)
         Qts = Qts + 0.5*Estep*(QQts+Qtslast)     ! for use in strong collider calculation
         Qlast = QQ
         Rlast = RR
         Qtslast=QQts
 100  CONTINUE
c
c          NOW CONTINUE TO HIGHER ENERGIES
c 
      Estep = 10.*Egrain1                          ! Step size (cm-1)
      Itop = ( 25.0/(hor*TT))/Estep               ! Integral to be extended to 200 kT; however the evaluation
      E2=E                                         ! will stop when Qcheck is less than Qthresh
      DO i = 1 , Itop
         E = E + Estep
         D = DENVR(Mol,E)
         QQ = D*exp(-E*hor*TT)                     ! continuation of molecule partition function
         Q  = Q  + 0.5*Estep*(QQ + Qlast)          ! Trapezoidal Rule
         Qlast = QQ
         Qcheck=dabs(QQ/Q)
         if(Qcheck.lt.Qthresh)then
           goto 17
         else
           continue
         end if

         if(E.gt.Emax2)then
           PAUSE 'Warning: Emax2 exceeded; extend Emax2 in Densum'
         else
           continue
         end if
       end do

 17     continue

       arg=E2+EC
       do i2=1,Itop
         arg = arg+Estep
         E2=E2+Estep
         D = DENVR(Mol,arg)
         CALL NTERP(E2,IE,X)
         Y1 = Rate(Mol,nchann,IE-1)
         Y2 = Rate(Mol,nchann,IE)
         Y3 = Rate(Mol,nchann,IE+1)
         CALL QUDINT(Y1,Y2,Y3,X,Y)                 ! Y = interpolated ln k(E)
         RR = D*exp(Y-E2*hor*TT)                   ! k(E) stored as log
         QQts = D*exp(-E2*hor*TT)                  ! continuation of partition function for active energy;
         RK = RK + 0.5*Estep*(RR + Rlast)
         Qts = Qts + 0.5*Estep*(QQts+Qtslast)      ! for use in strong collider calculation
         Rlast = RR
         Qtslast=QQts
         Qcheck=dabs(QQts/Qts)
          if(Qcheck.lt.Qthresh)then    
            goto 18
          else
            continue
          end if

          if(E2.gt.Emax2)then
            PAUSE 'Warning: Emax2 exceeded; extend Emax2 in Densum'
          else
            continue
          end if
        end do

 18    Continue

       RK = RK*exp(-Eo(Mol,nchann)*hor*TT)/Q    
       K(mm) = RK*(TSmom(Mol,nchann)/MolMom(Mol))     ! kinf with adiabatic Qrot ratio: equal to TST result
 
1000  CONTINUE

      Kinf = K(2)
      Et = -(LOG(K(3)/K(1)))/(TS(3)-TS(1))            ! Ea/R
      Ainf = K(2)*exp(Et*TS(2))                       ! 1/s
      Einf = (1.9872065d-03)*Et                       ! kcal/mole
C                                                                       strong collider low pressure rate constant
      kosc(Mol,nchann) = klj*Qts*exp(-Eo(Mol,nchann)*hor*TT)/Q        ! Based on Eq. 4.30 in Robinson & Holbrook
      kosc(Mol,nchann) =  kosc(Mol,nchann)*
     &                         (TSmom(Mol,nchann)/MolMom(Mol))

      RETURN
      END
