
      PROGRAM THERMO
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Thermo: a code for thermochemical calculations.
c Copyright (C) 2003, 2005 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c
c	Equilibrium constants and thermodynamic parameters
c	based on molecular properties
c
c	Last modified: 
c       05/05 (JRB) fixed conversion factors
c       04/05 (JRB) Added capability for Canonical TST
c       09/04 (JRB) Fixed bug that gave incorrect A,B in Kequil when MCC units
c       10/03 (JP) trimmed defined-length character strings with lenstr.
c       03/03 Incorporated unsymmetrical hindered internal rotation.
c       03/03 Refined output format
c       11/02 Fixed yet another bug related to vibrational degeneracies
c       3/02  Fixed bug that prevented correct treatment of vibrational
c             degeneracies.
c       2/02  Added particle in a Box
c       2/02  Output format changed in order to report zero point energies.
c       9/99 by J. R. Barker
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c	FORMAT FOR THERMO DATA FILE
c
c	LINE
c	1	Energy units Key-word	= 'CM-1', 'KCAL', or 'KJOU'
c		Std. State key-word	= 'ATM', 'BAR', 'MCC'
c					  (atmospheres, bars, molecules/cc)
c
c	2	Nt	= number of temperatures
c
c	3	List of temperatures
c
c	4	Ns	= number of species
c
c	REPEAT #5 and #6 Ns TIMES
c
c	5	REPROD	= key word:	'reac'	= reactant
c					'prod'	= product
c                                       'ctst'  = canonical transition state
c					'none'	= neither reactant nor product;
c					   not included in equilibrium constant
c		MOLNAME	= Name of chemical species (up to 10 characters)
c		DelH	= enthalpy of formation at 0 K
c		AMU	= molecular weight (g/mole)
c		Sym	= external symmetry number (INTEGER)
c		Sopt	= number of optical isomers (INTEGER)
c		Nele	= number of electronic levels (INTEGER)
c		N	= number of vibrations and rotations to be read in
c               VHAR    = 'HAR': for vibrational frequencies input as harmonic freqs.
c                       = 'OBS': for vibrational frequencies input as 0-1 observed freqs.
c               Elev,gele = (REAL,INTEGER; repeat Nele times) electronic level energy and 
C                           degeneracy. energies expressed in cm-1 units.
c                           NOTE: lowest level should be at Elev = 0.
c
c       6       TITLE = up to 50 character comment line
c
c	REPEAT N TIMES
c
c	7	MODE, IDOF, WE, ANH, NG
c
c		MODE	= index number for degree of freedom
c		IDOF	= type of degree of freedom key word
c		= 'vib' (harmonic or Morse Potential vibration)
c			WE	= vib frequency (cm-1)
c			ANH	= vib anharmonicity (cm-1)
c			NG	= vib degeneracy
c		= 'box' (particle in a box vibration)
c			WE	= vib frequency (cm-1)
c			NG	= vib degeneracy
c		= 'rot' (classical rotation)
c			WE	= rot moment of inertia (amu*ang^2)
c			ANH	= rot symmetry (foldedness)
c			NG	= rot dimension
c		= 'hra' (1-D symmetrical hindered rotor)
c			WE	= vib frequency (cm-1)
c			ANH	= reduced moment of inertia (amu*ang^2)
c			NG	= barrier foldedness (POSITIVE if equal to internal 
c                                 rotor symmetry number, NEGATIVE if different from
c                                 symmetry number)
c                         [NSIG = internal rotor symmetry number, is read only if 
c                                 NG is NEGATIVE]
c		= 'hrb' (1-D symmetrical hindered rotor)
c			WE	= vib frequency (cm-1)
c			ANH	= barrier (cm-1)
c			NG	= barrier foldedness (POSITIVE if equal to internal 
c                                 rotor symmetry number, NEGATIVE if different from
c                                 symmetry number)
c                         [NSIG = internal rotor symmetry number, is read only if 
c                                 NG is NEGATIVE]
c		= 'hrc' (1-D symmetrical hindered rotor)
c			WE	= reduced moment of inertia (amu*ang^2)
c			ANH	= barrier (cm-1)
c			NG	= barrier foldedness (POSITIVE if equal to internal 
c                                 rotor symmetry number, NEGATIVE if different from
c                                 symmetry number)
c                         [NSIG = internal rotor symmetry number, is read only if 
c                                 NG is NEGATIVE]
c
c	8	BLANK LINE
c

      include 'declare.inc' 

      Double Precision rmax0,I2dGorin0, Veff0
      

!     OPEN (UNIT=4,STATUS='OLD',FILE='thermo.dat')              ! General Data file  (read_dat.f)
!     OPEN (UNIT=3,STATUS='UNKNOWN',FILE='thermo.out')          ! Full output        (write_out.f)


      
      call read_dat                            ! READ THERMO.DAT AND CALCULATE ZPE

      DO Temp=1,Nt  ! Nt  number of temperatures
       
        if(Gorin.gt.0) then 
         call MaxVeff(Nt,beta,POT,c,De,re,mu,T(Temp),rmax0,I2dGorin0,V0,
     &Veff0,Eunits)                             ! I2d Gorin Subroutine

         V(Temp)=V0
         Veff(temp)=Veff0
         I2dGorin(Temp)=I2dGorin0
         bet(Temp)=beta
         rmax(Temp)=rmax0
        endif

        if(hindrance.gt.0) then 
           call hindrance_fit                         !  Hindrance calculation
        ELSE
           call thermochem                            !  Thermochemistry calculation
        ENDIF

           T_AK(Temp)= AK(2)
           T_AA(Temp)= AA
           T_BB(Temp)=BB
           T_DSR(Temp)=DSR
           T_DHR(Temp)=DHR
           T_DCp(Temp)=DCp
           T_Sfac(Temp)=Sfac
           do x=1,Ns
            T_S(Temp,x)=S(x)
            T_Cp(Temp,x)=Cp(x)
            T_H(Temp,x)=H(x)
           enddo
        
      ENDDO

      call write_out                                  ! Write thermo.out 
  

 
      END


